@@ -53,34 +53,34 @@ class NGramShrink : public NGramMutableModel<Arc> {
   using NGramMutableModel<Arc>::ScalarValue;
   using NGramMutableModel<Arc>::FindMutableArc;
 
-  // Construct an NGramShrink object, including an NGramModel and parameters
+  // Constructs an NGramShrink object, including an NGramModel and parameters.
   explicit NGramShrink(MutableFst<Arc> *infst, int shrink_opt = 0,
                        double tot_uni = -1.0, Label backoff_label = 0,
                        double norm_eps = kNormEps,
                        bool check_consistency = false, bool norm = true);
 
-  // Shrink n-gram model, based on initialized parameters
+  // Shrinks n-gram model, based on initialized parameters.
   bool ShrinkNGramModel(bool require_norm);
 
   // Calculates shrinking scores for all ngrams, without actually pruning (yet).
   void CalculateShrinkScores(bool require_norm);
 
-  // Provides ngram label vectors and/or vector of their shrink scores.
-  void GetNGramsAndOrScores(vector<vector<Label>> *ngrams,
-                            vector<double> *scores, bool collect_unigrams);
+  // Provides label vectors and/or vector of their shrink scores.
+  void GetNGramsAndOrScores(std::vector<std::vector<Label>> *ngrams,
+                            std::vector<double> *scores, bool collect_unigrams);
 
   virtual ~NGramShrink() {}
 
  protected:
-  // Data representation for an arc being considered for pruning
+  // Data representation for an arc being considered for pruning.
   struct ShrinkArcStats {
-    double log_prob;          // log probability of word given history
-    double log_backoff_prob;  // log probability of word given backoff history
-    double shrink_score;      // calculated score for shrinking.
-    Label label;              // arc label
-    StateId backoff_dest;     // destination state of backoff arc
-    bool needed;              // is the current arc needed within the automaton?
-    bool pruned;  // has the current arc been pruned already by shrinking?
+    double log_prob;          // Log probability of word given history.
+    double log_backoff_prob;  // Log probability of word given backoff history.
+    double shrink_score;      // Calculated score for shrinking.
+    Label label;              // Arc label.
+    StateId backoff_dest;     // Destination state of backoff arc.
+    bool needed;              // Is the current arc needed within the automaton?
+    bool pruned;  // Has the current arc been pruned already by shrinking?
 
     ShrinkArcStats(double lp, double lbp, Label lab, StateId dest, bool needed)
         : log_prob(lp),
@@ -92,20 +92,20 @@ class NGramShrink : public NGramMutableModel<Arc> {
           pruned(false) {}
   };
 
-  // Data representation for a state with arcs being considered for pruning
+  // Data representation for a state with arcs being considered for pruning.
   struct ShrinkStateStats {
-    double log_prob;        // log probability of history represented by state
-    StateId state;          // state ID of current state
-    StateId backoff_state;  // state ID of backoff state
-    StateId prefix_state;   // state ID of prior state on ascending path.
-    Label incoming_label;   // label of arc leading to state on ascending path.
-    bool state_dead;        // store whether state is to be removed from model
+    double log_prob;        // Log probability of history represented by state.
+    StateId state;          // State ID of current state.
+    StateId backoff_state;  // State ID of backoff state.
+    StateId prefix_state;   // State ID of prior state on ascending path.
+    Label incoming_label;   // Label of arc leading to state on ascending path.
+    bool state_dead;        // Store whether state is to be removed from model.
     // # of arcs that back off thru incoming arc. This is only for incoming
     // arcs that increase in state order and thus are uniquely determined
     // by their destination state.
-    // NB: dest. state uniquely determines arc label in this case.
+    // NB: destination state uniquely determines arc label in this case.
     size_t incoming_backed_off;
-    // # of final states that backoff to state
+    // # of final states that backoff to state.
     size_t incoming_st_back_off;
 
     ShrinkStateStats()
@@ -119,9 +119,9 @@ class NGramShrink : public NGramMutableModel<Arc> {
           incoming_st_back_off(0) {}
   };
 
-  // Provides the score provided to arc for particular shrinking method
-  // Need to override in derived class for anything but count pruning
-  // Default calculates count for normalized model; raw count for unnormalized
+  // Provides the score provided to arc for particular shrinking method. One
+  // must override this in any derived class for anything but count pruning.
+  // Default calculates count for normalized model; raw count for unnormalized.
   virtual double ShrinkScore(const ShrinkStateStats &state,
                              const ShrinkArcStats &arc) const {
     if (!normalized_) {
@@ -134,26 +134,26 @@ class NGramShrink : public NGramMutableModel<Arc> {
     return arc.log_prob + state.log_prob + log(total_unigram_count_);
   }
 
-  // Provides the threshold for comparing to the scores to decide to prune
-  // Required from derived classes
+  // Provides the threshold for comparing to the scores to decide to prune.
+  // Required from derived classes.
   virtual double GetTheta(StateId state) const = 0;
 
   // Returns the theta value that guarantees at most target_number_of_ngrams.
   double ThetaForMaxNGrams(int target_number_of_ngrams);
 
-  // Calculates the new backoff weight if arc removed
+  // Calculates the new backoff weight if arc removed.
   double CalcNewLogBackoff(const ShrinkArcStats &arc) const {
     return NegLogSum(nlog_backoff_denom_, -arc.log_backoff_prob) -
            NegLogSum(nlog_backoff_num_, -arc.log_prob);
   }
 
-  // Provides access to total unigram count
+  // Provides access to total unigram count.
   double GetTotalUnigramCount() const { return total_unigram_count_; }
 
-  // Provides access to negative log numerator of the backoff
+  // Provides access to negative log numerator of the backoff.
   double GetNLogBackoffNum() const { return nlog_backoff_num_; }
 
-  // Provides access to negative log denominator of the backoff
+  // Provides access to negative log denominator of the backoff.
   double GetNLogBackoffDenom() const { return nlog_backoff_denom_; }
 
  private:
@@ -165,21 +165,22 @@ class NGramShrink : public NGramMutableModel<Arc> {
     }
   };
 
-  // Fills ngram label vector in correct order via recursive function.
-  void AddStateNGramLabels(StateId st, vector<Label> *ngram_labels);
+  // Fills n-gram label vector in correct order via recursive function.
+  void AddStateNGramLabels(StateId st, std::vector<Label> *ngram_labels);
 
-  // Finds an entry in the map with shrink score or fatal error.
+  // Finds an entry in the map with shrink score or produces fatal error.
   double FindOrDieShrinkScore(StateId st, Label label);
 
-  // transition from 'st' to 'dest' labeled with 'label'.
+  // Transition from 'st' to 'dest' labeled with 'label'.
   size_t &BackedOffTo(StateId st, Label label, StateId dest) {
-    if (StateOrder(st) < StateOrder(dest))  // arc unique to dest; store there
-      return shrink_state_[dest].incoming_backed_off;
-    else                                            // o.w. hash it
-      return backed_off_to_[std::make_pair(st, label)];  // inserts if needed.
+    if (StateOrder(st) < StateOrder(dest)) {           // Arc unique
+      return shrink_state_[dest].incoming_backed_off;  // to dest., store there.
+    } else {                                             // o.w. hash it.
+      return backed_off_to_[std::make_pair(st, label)];  // Inserts if needed.
+    }
   }
 
-  // Efficiently checks if non-zero BackedOffTo() (no side-effects)
+  // Efficiently checks if non-zero BackedOffTo() (no side-effects).
   bool IsBackedOffTo(StateId st, Label label, StateId dest) const {
     if (StateOrder(st) < StateOrder(dest))
       return shrink_state_[dest].incoming_backed_off > 0;
@@ -192,16 +193,16 @@ class NGramShrink : public NGramMutableModel<Arc> {
     }
   }
 
-  // Fill in relevant statistics for arc pruning at the state level
+  // Fills in relevant statistics for arc pruning at the state level.
   void FillShrinkStateInfo();
 
-  // Adds probs to backoff numerator and denominator
+  // Adds probabilities to backoff numerator and denominator.
   void AddToBackoffNumDenom(double num_upd_val, double denom_upd_val) {
     nlog_backoff_num_ = NegLogSum(nlog_backoff_num_, num_upd_val);
     nlog_backoff_denom_ = NegLogSum(nlog_backoff_denom_, denom_upd_val);
   }
 
-  // Subtracts probs from backoff numerator and denominator
+  // Subtracts probabilities from backoff numerator and denominator.
   void UpdateBackoffNumDenom(double num_upd_val, double denom_upd_val,
                              double *neg_log_correct_num,
                              double *neg_log_correct_denom) {
@@ -211,32 +212,36 @@ class NGramShrink : public NGramMutableModel<Arc> {
         NegLogSum(nlog_backoff_denom_, denom_upd_val, neg_log_correct_denom);
   }
 
-  // Updates maximum score for a given label leaving a given state. Returns max.
+  // Updates maximum score for a given label leaving a given state, and returns
+  // the maximum.
   double UpdateScoreHash(StateId st, Label label, double shrink_score);
 
   // Retrieves shrink score, calculating if requested.
   double GetShrinkScore(const ShrinkArcStats &arc, StateId st, Label label,
                         bool calc_score);
 
-  // Calculate and store statistics for scoring arc in pruning
-  int AddArcStat(vector<ShrinkArcStats> *shrink_arcs, StateId st,
+  // Calculates and store statistics for scoring arc in pruning.
+  int AddArcStat(std::vector<ShrinkArcStats> *shrink_arcs, StateId st,
                  const Arc *arc, const Arc *barc, bool calc_score);
 
-  // Fill in relevant statistics for arc pruning for a particular state
-  size_t FillShrinkArcInfo(vector<ShrinkArcStats> *shrink_arcs, StateId st,
+  // Fills in relevant statistics for arc pruning for a particular state.
+  size_t FillShrinkArcInfo(std::vector<ShrinkArcStats> *shrink_arcs, StateId st,
                            bool calc_score);
 
   // Calculates scores of all arcs leaving all states in model.
   void ScoreAllArcs();
 
-  // Non-greedy comparison to threshold, such as used for count pruning
-  size_t ArcsToPrune(vector<ShrinkArcStats> *shrink_arcs, StateId st) const;
+  // Non-greedy comparison to threshold, such as used for count pruning.
+  size_t ArcsToPrune(std::vector<ShrinkArcStats> *shrink_arcs,
+                     StateId st) const;
 
-  // Evaluate arcs and select arcs to prune in greedy fashion
-  size_t GreedyArcsToPrune(vector<ShrinkArcStats> *shrink_arcs, StateId st);
+  // Evaluates arcs and select arcs to prune in greedy fashion.
+  size_t GreedyArcsToPrune(std::vector<ShrinkArcStats> *shrink_arcs,
+                           StateId st);
 
-  // Evaluate arcs and select arcs to prune
-  size_t ChooseArcsToPrune(vector<ShrinkArcStats> *shrink_arcs, StateId st) {
+  // Evaluates arcs and select arcs to prune.
+  size_t ChooseArcsToPrune(std::vector<ShrinkArcStats> *shrink_arcs,
+                           StateId st) {
     if (shrink_opt_ < 2)
       return ArcsToPrune(shrink_arcs, st);
     else
@@ -244,7 +249,8 @@ class NGramShrink : public NGramMutableModel<Arc> {
   }
 
   // For transitions selected to be pruned, point them to an unconnected state
-  size_t PointPrunedArcs(const vector<ShrinkArcStats> &shrink_arcs, StateId st);
+  size_t PointPrunedArcs(const std::vector<ShrinkArcStats> &shrink_arcs,
+                         StateId st);
 
   // Evaluate transitions from state and prune in greedy fashion
   void PruneState(StateId st);
@@ -273,13 +279,11 @@ class NGramShrink : public NGramMutableModel<Arc> {
   double nlog_backoff_denom_;   // denominator of backoff weight
   StateId ns_;                  // Original number of states in the model
   StateId dead_state_;  // Sink state dest. for pruned arcs (not connected)
-  vector<ShrinkStateStats> shrink_state_;
+  std::vector<ShrinkStateStats> shrink_state_;
   std::unordered_map<std::pair<StateId, Label>, double, StateLabelHash>
       max_shrink_score_;
   std::unordered_map<std::pair<StateId, Label>, size_t, StateLabelHash>
       backed_off_to_;
-
-  DISALLOW_COPY_AND_ASSIGN(NGramShrink);
 };
 
 // Construct an NGramShrink object, including an NGramMutableModel
@@ -308,7 +312,7 @@ void NGramShrink<Arc>::ScoreAllArcs() {
   for (int order = HiOrder(); order > 1; --order) {
     for (StateId st = 0; st < ns_; ++st) {
       if (StateOrder(st) == order) {  // current order
-        vector<ShrinkArcStats> shrink_arcs;
+        std::vector<ShrinkArcStats> shrink_arcs;
         FillShrinkArcInfo(&shrink_arcs, st, true);
         if (Error()) return;
       }
@@ -373,7 +377,7 @@ bool NGramShrink<Arc>::ShrinkNGramModel(bool require_norm) {
 
 template <class Arc>
 void NGramShrink<Arc>::FillStateProbs() {
-  vector<double> probs;
+  std::vector<double> probs;
   CalculateStateProbs(&probs);
   for (StateId st = 0; st < ns_; ++st)
     shrink_state_[st].log_prob = log(probs[st]);
@@ -443,14 +447,14 @@ double NGramShrink<Arc>::FindOrDieShrinkScore(StateId st, Label label) {
 // Provides ngram label vectors and/or vector of their shrink scores.
 template <class Arc>
 void NGramShrink<Arc>::GetNGramsAndOrScores(vector<vector<Label>> *ngrams,
-                                            vector<double> *scores,
+                                            std::vector<double> *scores,
                                             bool collect_unigrams) {
   if (ngrams == nullptr && scores == nullptr) return;
   for (StateId st = 0; st < ns_; ++st) {
-    vector<Label> state_ngram;
+    std::vector<Label> state_ngram;
     AddStateNGramLabels(st, &state_ngram);  // Labels of words leading to state.
     if (state_ngram.size() == 0 && !collect_unigrams) continue;
-    vector<Label> to_update;
+    std::vector<Label> to_update;
     for (ArcIterator<ExpandedFst<Arc>> aiter(GetExpandedFst(), st);
          !aiter.Done(); aiter.Next()) {
       Arc arc = aiter.Value();
@@ -462,7 +466,7 @@ void NGramShrink<Arc>::GetNGramsAndOrScores(vector<vector<Label>> *ngrams,
       to_update.push_back(kNoLabel);
     for (size_t idx = 0; idx < to_update.size(); ++idx) {
       if (ngrams != nullptr) {
-        vector<Label> ngram_labels = state_ngram;
+        std::vector<Label> ngram_labels = state_ngram;
         ngram_labels.push_back(to_update[idx]);
         ngrams->push_back(ngram_labels);
       }
@@ -596,7 +600,7 @@ double NGramShrink<Arc>::ThetaForMaxNGrams(int target_number_of_ngrams) {
     NGRAMERROR() << "ThetaForMaxNGrams: Error in calculating shrink scores";
     return 0.0;
   }
-  vector<double> scores;  // Only care about scores, not ngram identities.
+  std::vector<double> scores;  // Only care about scores, not ngram identities.
   NGramShrink<Arc>::GetNGramsAndOrScores(nullptr, &scores, false);
   if (Error()) {
     NGRAMERROR() << "ThetaForMaxNGrams: Error in getting ngram scores";
@@ -699,10 +703,10 @@ size_t NGramShrink<Arc>::PointPrunedArcs(
   return pruned_cnt;
 }
 
-// Evaluate transitions from state and prune in greedy fashion
+// Evaluates transitions from state and prune in greedy fashion.
 template <class Arc>
 void NGramShrink<Arc>::PruneState(StateId st) {
-  vector<ShrinkArcStats> shrink_arcs;
+  std::vector<ShrinkArcStats> shrink_arcs;
   size_t candidate_prune = FillShrinkArcInfo(&shrink_arcs, st, false);
   if (Error()) return;
   size_t pruned_cnt = ChooseArcsToPrune(&shrink_arcs, st);
@@ -719,7 +723,7 @@ void NGramShrink<Arc>::PruneState(StateId st) {
   }
 }
 
-// Find unpruned arcs pointing to unconnected states and point them elsewhere
+// Finds unpruned arcs pointing to unconnected states and points them elsewhere.
 template <class Arc>
 void NGramShrink<Arc>::PointArcsAwayFromDead() {
   for (StateId st = 0; st < ns_; ++st) {
@@ -738,7 +742,7 @@ void NGramShrink<Arc>::PointArcsAwayFromDead() {
   PointDeadBackoffArcs();
 }
 
-// Map backoff arcs of dead states to dead_state_ (except for start state)
+// Maps backoff arcs of dead states to dead_state_ (except for start state).
 template <class Arc>
 void NGramShrink<Arc>::PointDeadBackoffArcs() {
   for (StateId st = 0; st < ns_; ++st) {
@@ -756,6 +760,15 @@ void NGramShrink<Arc>::PointDeadBackoffArcs() {
   }
 }
 
+// Makes model from NGram model FST with StdArc counts.
+bool NGramShrinkModel(fst::StdMutableFst *fst, const string &method,
+                      double tot_uni = -1.0, double theta = 0.0,
+                      int64 target_num = -1, const string &count_pattern = "",
+                      const string &context_pattern = "", int shrink_opt = 0,
+                      fst::StdArc::Label backoff_label = 0,
+                      double norm_eps = kNormEps,
+                      bool check_consistency = false);
+
 }  // namespace ngram
 
 #endif  // NGRAM_NGRAM_SHRINK_H_
