@@ -15,16 +15,14 @@
 #ifndef NGRAM_UTIL_H_
 #define NGRAM_UTIL_H_
 
-#include <fst/types.h>
+#include <fst/log.h>
+#include <fst/flags.h>
 
-//
 // UTILITY FOR ERROR HANDLING
-//
 
 DECLARE_bool(ngram_error_fatal);
 
 #define NGRAMERROR() \
-  LOG(LEVEL(FLAGS_ngram_error_fatal ? \
-            base_logging::FATAL : base_logging::ERROR))
+  (FLAGS_ngram_error_fatal ? LOG(FATAL) : LOG(ERROR))
 
 #endif  // NGRAM_UTIL_H_
