@@ -5,18 +5,19 @@
 bin=../bin
 testdata=$srcdir/testdata
 tmpdata=${TMPDIR:-/tmp}
-tmpprefix="${tmpdata}/ngramperp-earnest"
+tmpsuffix="$(mktemp -u XXXXXXXX 2>/dev/null)"
+tmpprefix="${tmpdata}/ngramperp-earnest-$tmpsuffix-$RANDOM-$$"
 
 trap "rm -f ${tmpprefix}*" 0 2 13 15
 
 set -e
 compile_test_fst() {
-  if [ ! -e "${tmpdata}/ngramperp-${1}.ref" ]
+  if [ ! -e "${tmpprefix}-${1}.ref" ]
   then
     fstcompile \
       -isymbols="${testdata}/${1}.sym" -osymbols="${testdata}/${1}.sym" \
       -keep_isymbols -keep_osymbols -keep_state_numbering \
-      "${testdata}/${1}.txt" "${tmpdata}/ngramperp-${1}.ref"
+      "${testdata}/${1}.txt" "${tmpprefix}-${1}.ref"
   fi
 }
 
@@ -27,7 +28,8 @@ farcompilestrings \
 
 compile_test_fst earnest-witten_bell.mod
 "${bin}/ngramperplexity" --OOV_probability=0.01 \
-  "${tmpprefix}"-witten_bell.mod.ref "${tmpprefix}".far "${tmpprefix}".perp
+  "${tmpprefix}"-earnest-witten_bell.mod.ref \
+  "${tmpprefix}".far "${tmpprefix}".perp
 
 cmp "${testdata}"/earnest.perp "${tmpprefix}".perp
 
