@@ -5,18 +5,19 @@
 bin=../bin
 testdata=$srcdir/testdata
 tmpdata=${TMPDIR:-/tmp}
-tmpprefix="${tmpdata}/ngramshrink-earnest"
+tmpsuffix="$(mktemp -u XXXXXXXX 2>/dev/null)"
+tmpprefix="${tmpdata}/ngramshrink-earnest-$tmpsuffix-$RANDOM-$$"
 
 trap "rm -f ${tmpprefix}*" 0 2 13 15
 
 set -e
 compile_test_fst() {
-  if [ ! -e "${tmpdata}/ngramshrink-${1}.ref" ]
+  if [ ! -e "${tmpprefix}-${1}.ref" ]
   then
     fstcompile \
       -isymbols="${testdata}/${1}.sym" -osymbols="${testdata}/${1}.sym" \
       -keep_isymbols -keep_osymbols -keep_state_numbering \
-      "${testdata}/${1}.txt" "${tmpdata}/ngramshrink-${1}.ref"
+      "${testdata}/${1}.txt" "${tmpprefix}-${1}.ref"
   fi
 }
 
@@ -31,10 +32,10 @@ do
 
   compile_test_fst "earnest-${method}.pru"
   "${bin}/ngramshrink" --method="${method}" --check_consistency "${param}" \
-    "${tmpprefix}-witten_bell.mod.ref" "${tmpprefix}-${method}.pru"
+    "${tmpprefix}-earnest-witten_bell.mod.ref" "${tmpprefix}-${method}.pru"
 
   fstequal \
-    "${tmpprefix}-${method}.pru.ref" "${tmpprefix}-${method}.pru"
+    "${tmpprefix}-earnest-${method}.pru.ref" "${tmpprefix}-${method}.pru"
 done
 
 for method in relative_entropy seymore
@@ -46,10 +47,11 @@ do
 
   "${bin}/ngramshrink" --method="${method}" --check_consistency \
     --target_number_of_ngrams="${target}" \
-    "${tmpprefix}-witten_bell.mod.ref" "${tmpprefix}-${method}.target.pru"
+    "${tmpprefix}-earnest-witten_bell.mod.ref" \
+    "${tmpprefix}-${method}.target.pru"
 
   fstequal \
-    "${tmpprefix}-${method}.pru.ref" "${tmpprefix}-${method}.target.pru"
+    "${tmpprefix}-earnest-${method}.pru.ref" "${tmpprefix}-${method}.target.pru"
 done
 
 echo PASS
